@@ -1,156 +1,58 @@
+#![allow(clippy::module_name_repetitions)]
 mod builder;
 mod config;
 mod state;
-pub use self::builder::{FileLogWriterBuilder, FlWriteMode};
+mod state_handle;
 
-use self::config::{Config, RotationConfig};
-use crate::primary_writer::buffer_with;
-use crate::writers::LogWriter;
-use crate::{DeferredNow, FileSpec, FlexiLoggerError, FormatFunction};
-#[cfg(feature = "async")]
-use crossbeam::{
-    channel::{self, SendError, Sender},
-    queue::ArrayQueue,
+pub use self::builder::{ArcFileLogWriter, FileLogWriterBuilder, FileLogWriterHandle};
+
+use self::{
+    config::{Config, RotationConfig},
+    state::State,
+    state_handle::StateHandle,
+};
+use crate::{
+    writers::LogWriter, DeferredNow, EffectiveWriteMode, FileSpec, FlexiLoggerError, FormatFunction,
 };
 use log::Record;
-use state::State;
-use std::io::Write;
 use std::path::PathBuf;
-#[cfg(feature = "async")]
-use std::sync::Arc;
-use std::sync::Mutex;
-#[cfg(feature = "async")]
-use std::thread::JoinHandle;
 
 const WINDOWS_LINE_ENDING: &[u8] = b"\r\n";
 const UNIX_LINE_ENDING: &[u8] = b"\n";
-#[cfg(feature = "async")]
-const ASYNC_FLW_FLUSH: &[u8] = b"F";
-#[cfg(feature = "async")]
-const ASYNC_FLW_SHUTDOWN: &[u8] = b"S";
-
-const ERR_1: &str = "FileLogWriter: formatting failed ";
-const ERR_2: &str = "FileLogWriter: writing failed ";
-#[cfg(feature = "async")]
-const ERR_3: &str = "FileLogWriter: flushing failed ";
-
-fn write_err(msg: &str, err: &std::io::Error) {
-    eprintln!("[flexi_logger] {} with {}", msg, err);
-}
 
 /// A configurable [`LogWriter`] implementation that writes to a file or a sequence of files.
 ///
 /// See [writers](crate::writers) for usage guidance.
 pub struct FileLogWriter {
-    format: FormatFunction,
     // the state needs to be mutable; since `Log.log()` requires an unmutable self,
     // which translates into a non-mutating `LogWriter::write()`,
     // we need internal mutability and thread-safety.
     state_handle: StateHandle,
     max_log_level: log::LevelFilter,
 }
-enum StateHandle {
-    Sync(Mutex<State>),
-    #[cfg(feature = "async")]
-    Async(AsyncHandle),
-}
-impl std::fmt::Debug for StateHandle {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> Result<(), std::fmt::Error> {
-        match self {
-            Self::Sync(ref m_state) => f.write_fmt(format_args!("{:?}", &*m_state.lock().unwrap())),
-            #[cfg(feature = "async")]
-            Self::Async(ref async_handle) => f.write_fmt(format_args!("{:?}", async_handle)),
-        }
-    }
-}
-
-#[cfg(feature = "async")]
-#[derive(Debug)]
-struct AsyncHandle {
-    am_state: Arc<Mutex<State>>,
-    sender: Sender<Vec<u8>>,
-    mo_thread_handle: Mutex<Option<JoinHandle<()>>>,
-    a_pool: Arc<ArrayQueue<Vec<u8>>>,
-    msg_capa: usize,
-}
-#[cfg(feature = "async")]
-impl AsyncHandle {
-    fn pop_buffer(&self) -> Vec<u8> {
-        self.a_pool
-            .pop()
-            .unwrap_or_else(|| Vec::with_capacity(self.msg_capa))
-    }
-    fn send(&self, buffer: Vec<u8>) -> Result<(), SendError<Vec<u8>>> {
-        self.sender.send(buffer)
-    }
-}
-
 impl FileLogWriter {
     pub(crate) fn new(
-        format: FormatFunction,
         state: State,
         max_log_level: log::LevelFilter,
+        format_function: FormatFunction,
     ) -> FileLogWriter {
-        #[cfg(feature = "async")]
-        let state_handle = if let FlWriteMode::BufferAsync(_bufsize, pool_capa, msg_capa) =
-            state.config().write_mode
-        {
-            let am_state = Arc::new(Mutex::new(state));
-            let (sender, receiver) = channel::unbounded::<Vec<u8>>();
-            let a_pool = Arc::new(ArrayQueue::new(pool_capa));
+        let state_handle = match state.config().write_mode.inner() {
+            EffectiveWriteMode::Direct
+            | EffectiveWriteMode::BufferAndFlushWith(_, _)
+            | EffectiveWriteMode::BufferDontFlushWith(_) => {
+                StateHandle::new_sync(state, format_function)
+            }
 
-            let t_state = Arc::clone(&am_state);
-            let t_pool = Arc::clone(&a_pool);
-
-            let mo_thread_handle = Mutex::new(Some(
-                std::thread::Builder::new()
-                    .name("flexi_logger-async_file_log_writer".to_string())
-                    .spawn(move || loop {
-                        match receiver.recv() {
-                            Err(_) => break,
-                            Ok(mut message) => {
-                                let mut state = t_state.lock().unwrap();
-                                match message.as_ref() {
-                                    ASYNC_FLW_FLUSH => {
-                                        state.flush().unwrap_or_else(|e| write_err(ERR_3, &e));
-                                    }
-                                    ASYNC_FLW_SHUTDOWN => {
-                                        state.shutdown();
-                                        break;
-                                    }
-                                    _ => {
-                                        message
-                                            .write_all(state.config().line_ending)
-                                            .unwrap_or_else(|e| write_err(ERR_2, &e));
-                                        state
-                                            .write_buffer(&message)
-                                            .unwrap_or_else(|e| write_err(ERR_2, &e));
-                                    }
-                                }
-                                if message.capacity() <= msg_capa {
-                                    message.clear();
-                                    t_pool.push(message).ok();
-                                }
-                            }
-                        }
-                    })
-                    .unwrap(),
-            )); // yes, let's panic if the thread can't be spawned
-            StateHandle::Async(AsyncHandle {
-                am_state,
-                sender,
-                mo_thread_handle,
-                a_pool,
-                msg_capa,
-            })
-        } else {
-            StateHandle::Sync(Mutex::new(state))
+            #[cfg(feature = "async")]
+            EffectiveWriteMode::AsyncWith {
+                bufsize: _,
+                pool_capa,
+                message_capa,
+                flush_interval: _,
+            } => StateHandle::new_async(pool_capa, message_capa, state, format_function),
         };
-        #[cfg(not(feature = "async"))]
-        let state_handle = StateHandle::Sync(Mutex::new(state));
 
         FileLogWriter {
-            format,
             state_handle,
             max_log_level,
         }
@@ -163,19 +65,20 @@ impl FileLogWriter {
     }
 
     /// Returns a reference to its configured output format function.
+    #[must_use]
     #[inline]
     pub fn format(&self) -> FormatFunction {
-        self.format
+        self.state_handle.format_function()
     }
 
+    #[must_use]
     #[doc(hidden)]
     pub fn current_filename(&self) -> PathBuf {
-        match &self.state_handle {
-            StateHandle::Sync(state) => state.lock().unwrap(),
-            #[cfg(feature = "async")]
-            StateHandle::Async(handle) => handle.am_state.lock().unwrap(),
-        }
-        .current_filename()
+        self.state_handle.current_filename()
+    }
+
+    pub(crate) fn plain_write(&self, buffer: &[u8]) -> std::result::Result<usize, std::io::Error> {
+        self.state_handle.plain_write(buffer)
     }
 
     /// Replaces parts of the configuration of the file log writer.
@@ -191,83 +94,19 @@ impl FileLogWriter {
     /// `FlexiLoggerError::Io` if the specified path doesn't work.
     /// `FlexiLoggerError::Poison` if some mutex is poisoned.
     pub fn reset(&self, flwb: &FileLogWriterBuilder) -> Result<(), FlexiLoggerError> {
-        let mut state = match &self.state_handle {
-            StateHandle::Sync(state) => state.lock(),
-            #[cfg(feature = "async")]
-            StateHandle::Async(handle) => handle.am_state.lock(),
-        }
-        .map_err(|_| FlexiLoggerError::Poison)?;
-        flwb.assert_write_mode((*state).config().write_mode)?;
-        *state = flwb.try_build_state()?;
-        Ok(())
+        self.state_handle.reset(flwb)
     }
 }
 
 impl LogWriter for FileLogWriter {
     #[inline]
     fn write(&self, now: &mut DeferredNow, record: &Record) -> std::io::Result<()> {
-        match &self.state_handle {
-            StateHandle::Sync(state) => {
-                buffer_with(|tl_buf| match tl_buf.try_borrow_mut() {
-                    Ok(mut buffer) => {
-                        (self.format)(&mut *buffer, now, record)
-                            .unwrap_or_else(|e| write_err(ERR_1, &e));
-                        let mut state_guard = state.lock().unwrap();
-                        let state = &mut *state_guard;
-                        buffer
-                            .write_all(state.config().line_ending)
-                            .unwrap_or_else(|e| write_err(ERR_2, &e));
-                        state
-                            .write_buffer(&*buffer)
-                            .unwrap_or_else(|e| write_err(ERR_2, &e));
-                        buffer.clear();
-                    }
-                    Err(_e) => {
-                        // We arrive here in the rare cases of recursive logging
-                        // (e.g. log calls in Debug or Display implementations)
-                        // we print the inner calls, in chronological order, before finally the
-                        // outer most message is printed
-                        let mut tmp_buf = Vec::<u8>::with_capacity(200);
-                        (self.format)(&mut tmp_buf, now, record)
-                            .unwrap_or_else(|e| write_err(ERR_1, &e));
-                        let mut state_guard = state.lock().unwrap();
-                        let state = &mut *state_guard;
-                        tmp_buf
-                            .write_all(state.config().line_ending)
-                            .unwrap_or_else(|e| write_err(ERR_2, &e));
-                        state
-                            .write_buffer(&tmp_buf)
-                            .unwrap_or_else(|e| write_err(ERR_2, &e));
-                    }
-                });
-            }
-            #[cfg(feature = "async")]
-            StateHandle::Async(handle) => {
-                let mut buffer = handle.pop_buffer();
-                (self.format)(&mut buffer, now, record).unwrap_or_else(|e| write_err(ERR_1, &e));
-                handle.send(buffer).unwrap();
-            }
-        }
-
-        Ok(())
+        self.state_handle.write(now, record)
     }
 
     #[inline]
     fn flush(&self) -> std::io::Result<()> {
-        match &self.state_handle {
-            StateHandle::Sync(state) => {
-                if let Ok(ref mut state) = state.lock() {
-                    state.flush()?;
-                }
-            }
-            #[cfg(feature = "async")]
-            StateHandle::Async(handle) => {
-                let mut buffer = handle.pop_buffer();
-                buffer.extend(ASYNC_FLW_FLUSH);
-                handle.send(buffer).ok();
-            }
-        }
-        Ok(())
+        self.state_handle.flush()
     }
 
     #[inline]
@@ -277,39 +116,11 @@ impl LogWriter for FileLogWriter {
 
     #[doc(hidden)]
     fn validate_logs(&self, expected: &[(&'static str, &'static str, &'static str)]) {
-        match &self.state_handle {
-            StateHandle::Sync(state) => {
-                if let Ok(ref mut state) = state.lock() {
-                    state.validate_logs(expected)
-                }
-            }
-            #[cfg(feature = "async")]
-            StateHandle::Async(handle) => {
-                if let Ok(ref mut state) = handle.am_state.lock() {
-                    state.validate_logs(expected)
-                }
-            }
-        }
+        self.state_handle.validate_logs(expected);
     }
 
     fn shutdown(&self) {
-        match &self.state_handle {
-            StateHandle::Sync(state) => {
-                // do nothing in case of poison errors
-                if let Ok(ref mut state) = state.lock() {
-                    state.shutdown();
-                }
-            }
-            #[cfg(feature = "async")]
-            StateHandle::Async(handle) => {
-                let mut buffer = handle.pop_buffer();
-                buffer.extend(ASYNC_FLW_SHUTDOWN);
-                handle.send(buffer).unwrap();
-                if let Ok(ref mut o_th) = handle.mo_thread_handle.lock() {
-                    o_th.take().and_then(|th| th.join().ok());
-                }
-            }
-        }
+        self.state_handle.shutdown();
     }
 }
 
@@ -327,13 +138,13 @@ impl Drop for FileLogWriter {
 
 #[cfg(test)]
 mod test {
-    use crate::writers::FlWriteMode;
     use crate::writers::LogWriter;
-    use crate::{Cleanup, Criterion, DeferredNow, FileSpec, Naming};
+    use crate::{Cleanup, Criterion, DeferredNow, FileSpec, Naming, WriteMode};
     use chrono::Local;
 
     use std::ops::Add;
     use std::path::{Path, PathBuf};
+    use std::time::Duration;
     const DIRECTORY: &str = r"log_files/rotate";
     const ONE: &str = "ONE";
     const TWO: &str = "TWO";
@@ -461,13 +272,13 @@ mod test {
         assert!(list_rotated_files(&basename, &ts).is_empty());
         assert!(contains("CURRENT", &ts, ONE));
 
-        std::thread::sleep(std::time::Duration::from_secs(2));
+        std::thread::sleep(Duration::from_secs(2));
         // ensure this produces ONE/-/TWO
         write_loglines(false, naming, &ts, &[TWO]);
         assert_eq!(list_rotated_files(&basename, &ts).len(), 1);
         assert!(contains("CURRENT", &ts, TWO));
 
-        std::thread::sleep(std::time::Duration::from_secs(2));
+        std::thread::sleep(Duration::from_secs(2));
         // ensure this produces ONE/TWO/THREE
         write_loglines(false, naming, &ts, &[THREE]);
         assert_eq!(list_rotated_files(&basename, &ts).len(), 2);
@@ -530,7 +341,12 @@ mod test {
             .o_append(false);
 
             #[cfg(feature = "async")]
-            let flwb = flwb.write_mode(FlWriteMode::BufferAsync(5, 5, 400));
+            let flwb = flwb.write_mode(WriteMode::AsyncWith {
+                bufsize: 5,
+                pool_capa: 5,
+                message_capa: 400,
+                flush_interval: Duration::from_secs(0),
+            });
 
             let flw = flwb.try_build().unwrap();
 
@@ -553,7 +369,7 @@ mod test {
         }
 
         // give the cleanup thread a short moment of time
-        std::thread::sleep(std::time::Duration::from_millis(50));
+        std::thread::sleep(Duration::from_millis(50));
 
         let fn_pattern = String::with_capacity(180)
             .add(
@@ -579,10 +395,15 @@ mod test {
 
     #[test]
     fn test_reset() {
-        #[cfg(feature = "async")]
-        let flwrite_mode = FlWriteMode::BufferAsync(6, 7, 8);
         #[cfg(not(feature = "async"))]
-        let flwrite_mode = FlWriteMode::Buffer(4);
+        let write_mode = WriteMode::BufferDontFlushWith(4);
+        #[cfg(feature = "async")]
+        let write_mode = WriteMode::AsyncWith {
+            bufsize: 6,
+            pool_capa: 7,
+            message_capa: 8,
+            flush_interval: Duration::from_secs(0),
+        };
         let flw = super::FileLogWriter::builder(
             FileSpec::default()
                 .directory(DIRECTORY)
@@ -594,7 +415,7 @@ mod test {
             Cleanup::KeepLogFiles(20),
         )
         .append()
-        .write_mode(flwrite_mode)
+        .write_mode(write_mode)
         .try_build()
         .unwrap();
 
@@ -624,7 +445,7 @@ mod test {
                 Naming::Numbers,
                 Cleanup::KeepLogFiles(20),
             )
-            .write_mode(flwrite_mode),
+            .write_mode(write_mode),
         )
         .unwrap();
         flw.write(
@@ -653,7 +474,7 @@ mod test {
                     Naming::Numbers,
                     Cleanup::KeepLogFiles(20),
                 )
-                .write_mode(FlWriteMode::DontBuffer),
+                .write_mode(WriteMode::Direct),
             )
             .is_err());
     }
